"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SegmentedView = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactNative = require("react-native");

var _MaterialTabBar = require("./MaterialTabBar");

var _SegmentContext = require("./SegmentContext");

var _SegmentedControl = require("./SegmentedControl");

var _SegmentedViewContext = require("./SegmentedViewContext");

var _helpers = require("./helpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Basic usage looks like this:
 *
 * ```tsx
 * import { Segmented } from 'react-native-collapsible-segmented-view'
 *
 * const Example = () => {
 *   return (
 *     <Segmented.View hader={MyHeader}>
 *       <Segmented.Segment label="A" component={ScreenA} />
 *       <Segmented.Segment label="B" component={ScreenB} />
 *        <Segmented.Segment label="C" component={ScreenC} />
 *     </Tabs.Container>
 *   )
 * }
 * ```
 */
const SegmentedView = ({
  initialIndex = 0,
  animatedValue,
  headerHeight,
  controlHeight = _helpers.CONTROL_HEIGHT,
  containerHeight = 0,
  children,
  header: HeaderComponent,
  control: ControlComponent = _helpers.IS_IOS ? _SegmentedControl.SegmentedControl : _MaterialTabBar.MaterialTabBar,
  lazy = false,
  containerStyle,
  topStyle,
  disableFadeIn = false
}) => {
  const [labels] = _react.default.useState((0, _helpers.extractLabels)(children));

  const refs = _react.default.useRef(labels.map(() => undefined));
  /**
   * keep all heights on a single object insted of 3. This helps
   * reduce the rerenders from to 3 to 1 after the first mount
   */


  const [layoutHeights, setLayoutHeights] = _react.default.useState({
    header: headerHeight || 0,
    control: controlHeight,
    container: containerHeight,
    contentInset: _helpers.IS_IOS ? (headerHeight || 0) + controlHeight : 0
  });

  const trackHeaderHeight = _react.default.useRef(layoutHeights.header);

  const trackControlHeight = _react.default.useRef(layoutHeights.control);

  const trackContainerHeight = _react.default.useRef(layoutHeights.container);
  /**
   * used to fade in the content, after getting all
   * layout heights, if headerHeight is undefined
   */


  const onLayoutCalls = _react.default.useRef(0);

  const [scenesOpacity] = _react.default.useState(new _reactNative.Animated.Value(headerHeight === undefined ? 0 : 1));

  const [index] = _react.default.useState(new _reactNative.Animated.Value(initialIndex));

  const [floatIndex] = _react.default.useState(new _reactNative.Animated.Value(initialIndex));

  const trackIndex = _react.default.useRef(initialIndex);

  const prevIndex = _react.default.useRef(initialIndex);

  const offsets = _react.default.useRef(labels.map(() => -layoutHeights.contentInset));

  const [scrollY] = _react.default.useState(animatedValue || new _reactNative.Animated.Value(-layoutHeights.contentInset));

  const translateY = _react.default.useRef(scrollY.interpolate({
    inputRange: [0 - layoutHeights.contentInset, layoutHeights.header - layoutHeights.contentInset],
    outputRange: [0, -layoutHeights.header],
    extrapolate: 'clamp'
  }));

  const [visibility] = _react.default.useState(labels.map((_, i) => ({
    opacity: new _reactNative.Animated.Value(initialIndex === i ? 1 : 0),
    zIndex: new _reactNative.Animated.Value(initialIndex === i ? 1 : 0)
  })));

  const maybeTriggerRerenderAfterOnLayout = _react.default.useCallback(() => {
    // layout calls = hedaer + control + container
    if (onLayoutCalls.current >= 3) {
      const contentInset = _helpers.IS_IOS ? trackHeaderHeight.current + trackControlHeight.current : 0; // update translateY if the header height has changed

      if (trackHeaderHeight.current !== layoutHeights.header) {
        translateY.current = scrollY.interpolate({
          inputRange: [0 - contentInset, trackHeaderHeight.current - contentInset],
          outputRange: [0, -trackHeaderHeight.current],
          extrapolate: 'clamp'
        });
      } // update offsets and scrollY
      // to get correct value after first render


      if (onLayoutCalls.current === 3 && contentInset !== layoutHeights.contentInset) {
        offsets.current = offsets.current.map(() => -contentInset);
        scrollY.setValue(-contentInset);
      } // update the layoutHeights


      if (trackContainerHeight.current !== layoutHeights.container || trackHeaderHeight.current !== layoutHeights.header || trackControlHeight.current !== layoutHeights.control) {
        setLayoutHeights({
          header: trackHeaderHeight.current,
          control: trackControlHeight.current,
          container: trackContainerHeight.current,
          contentInset
        });
      } // fade in content in case it's hidden


      (0, _helpers.spring)(scenesOpacity, 1).start();
    }
  }, [layoutHeights.container, layoutHeights.contentInset, layoutHeights.control, layoutHeights.header, scenesOpacity, scrollY]);

  const onLayout = _react.default.useCallback((event, ref) => {
    onLayoutCalls.current += 1;
    const {
      height
    } = event.nativeEvent.layout;
    ref.current = height;
    maybeTriggerRerenderAfterOnLayout();
  }, [maybeTriggerRerenderAfterOnLayout]);

  const onContainerLayout = _react.default.useCallback(event => {
    onLayout(event, trackContainerHeight);
  }, [onLayout]);

  const onHeaderLayout = _react.default.useCallback(event => {
    onLayout(event, trackHeaderHeight);
  }, [onLayout]);

  const onControlLayout = _react.default.useCallback(event => {
    onLayout(event, trackControlHeight);
  }, [onLayout]);

  _react.default.useEffect(() => {
    offsets.current.forEach((_, index) => {
      scrollY.addListener(({
        value
      }) => {
        if (index === trackIndex.current) {
          offsets.current[index] = value;
        }
      });
    });
    return () => {
      scrollY.removeAllListeners();
    };
  }, [scrollY]);

  const setIndex = _react.default.useCallback((nextIndex, _currentIndex, _syncOnly) => {
    const currentIndex = _currentIndex === undefined ? trackIndex.current : _currentIndex;

    if (nextIndex !== currentIndex) {
      var _refs$current$nextInd;

      const currOffset = offsets.current[currentIndex];
      const nextOffset = offsets.current[nextIndex];
      let nextPosition = null; // compute how much we need to scroll to sync the next scene

      if (currOffset > nextOffset && nextOffset < layoutHeights.header) {
        nextPosition = Math.min(currOffset, layoutHeights.header);
      } else if (currOffset < nextOffset && currOffset < layoutHeights.header) {
        nextPosition = currOffset;
      }

      const nextRef = (_refs$current$nextInd = refs.current[nextIndex]) === null || _refs$current$nextInd === void 0 ? void 0 : _refs$current$nextInd.current; // scroll to the correct offset

      if (nextPosition !== null && nextRef) {
        (0, _helpers.scrollTo)(nextRef, nextPosition);
        offsets.current[nextIndex] = nextPosition;
      } // scroll to the top if is refrehing the current tab on iOS
      // before changing tabs


      const isRefreshingOnIOS = _helpers.IS_IOS && currOffset < -layoutHeights.contentInset;

      if (isRefreshingOnIOS) {
        var _refs$current$current;

        const ref = (_refs$current$current = refs.current[currentIndex]) === null || _refs$current$current === void 0 ? void 0 : _refs$current$current.current;
        ref && (0, _helpers.scrollTo)(ref, -layoutHeights.contentInset);
      }

      if (!_syncOnly) {
        // show the next scene, and hide the current one
        visibility[nextIndex].zIndex.setValue(2);
        visibility[currentIndex].opacity.setValue(0);
        visibility[currentIndex].zIndex.setValue(0);

        if (disableFadeIn) {
          visibility[nextIndex].opacity.setValue(1);
          visibility[nextIndex].zIndex.setValue(1);
        } else {
          (0, _helpers.spring)(visibility[nextIndex].opacity, 1).start(() => {
            visibility[nextIndex].zIndex.setValue(1);
          });
        } // update the mutable objects


        trackIndex.current = nextIndex;
        prevIndex.current = currentIndex; // update the animated values

        index.setValue(nextIndex);
        (0, _helpers.spring)(floatIndex, nextIndex).start();
      }
    }
  }, [layoutHeights.header, layoutHeights.contentInset, visibility, disableFadeIn, index, floatIndex]);

  const setRef = _react.default.useCallback((ref, index) => {
    refs.current[index] = ref;
  }, []);

  return /*#__PURE__*/_react.default.createElement(_SegmentedViewContext.SegmentedViewContext.Provider, {
    value: {
      contentInset: layoutHeights.contentInset,
      headerHeight: layoutHeights.header,
      controlHeight: layoutHeights.control,
      containerHeight: layoutHeights.container,
      scrollY,
      setRef,
      lazy,
      index,
      initialIndex,
      setIndex,
      prevIndex,
      trackIndex,
      translateY: translateY.current
    }
  }, /*#__PURE__*/_react.default.createElement(_reactNative.View, {
    style: [styles.container, containerStyle],
    onLayout: onContainerLayout,
    pointerEvents: "box-none"
  }, /*#__PURE__*/_react.default.createElement(_reactNative.Animated.View, {
    style: [styles.top, {
      transform: [{
        translateY: translateY.current
      }]
    }, topStyle],
    pointerEvents: "box-none"
  }, /*#__PURE__*/_react.default.createElement(_reactNative.View, {
    onLayout: onHeaderLayout,
    pointerEvents: "box-none"
  }, HeaderComponent && /*#__PURE__*/_react.default.createElement(HeaderComponent, null)), /*#__PURE__*/_react.default.createElement(_reactNative.View, {
    onLayout: onControlLayout,
    style: styles.control
  }, /*#__PURE__*/_react.default.createElement(ControlComponent, {
    index: index,
    floatIndex: floatIndex,
    setIndex: setIndex,
    initialIndex: initialIndex,
    labels: labels
  }))), /*#__PURE__*/_react.default.createElement(_reactNative.Animated.View, {
    style: [styles.scenes, {
      opacity: scenesOpacity
    }]
  }, _react.default.Children.map(children, (child, index) => /*#__PURE__*/_react.default.createElement(_SegmentContext.SegmentContext.Provider, {
    value: { ...visibility[index],
      index
    }
  }, child)))));
};

exports.SegmentedView = SegmentedView;

const styles = _reactNative.StyleSheet.create({
  container: {
    flex: 1
  },
  top: {
    width: '100%',
    position: 'absolute',
    zIndex: 100,
    backgroundColor: 'white',
    ..._reactNative.Platform.select({
      android: {
        elevation: 4
      }
    })
  },
  control: {
    zIndex: 100,
    width: '100%'
  },
  scenes: {
    flex: 1
  }
});
//# sourceMappingURL=SegmentedView.js.map